C     CONNECT
C
C     Satellite program for GRINSP
C     Geometrically Restrained INorganic Structure Prediction
C
C           Compares a .con file to a connectivity.txt file
C                  and detects the new frameworks which are  
C                  gathered in a file *-new.con
C                  summary in a *.txt file
C
C           takes account of R and symmetry (by SG number)
C                 selects the model with best R and highest
C                 SG number
C
C         - February 2005
C         - updated December 2005
C
C         - discard cases with nodes > 10 (bug in GRINSP)
C 
C         - care to problems if ndat >2000 and if more than 100
C                      identical solutions...
C
C
C   Armel Le Bail  alb@cristal.org
C
C=====================================================================
C
      program connect
C
C   ngood = max number of good solutions retained (10000)
C   nze = max number of known coordination sequences (5000)
C
      parameter (ngood=10000,nze=5000,nat=64)
      integer  mntype(ngood),mns(ngood,200),nzeot(nze)
      integer  nmzeot(nze,nat),nmzeotc(nze,nat),ll(ngood)
      integer  nzeotc(nze),lm(nze),lmc(nze),nsg(nze),nsgc(nze)
      integer  nsgs(2000,200),ns(2000)
      real r(nze),rc(nze),rd(ngood),rs(2000,200)
      character*32  zeot(nze),zeotc(nze),zeots(2000,200)
      character*40  coseq(nat,nze),coseqc(nat,nze),newcos(nat,ngood)
      character*4   t(10),version,atyp(2,2),btyp(2,2),fend
      character*20  file
      character*24  tempo
      character*10  sgn(nze),sgnc(ngood),sgns(2000,200)
	logical qex
C
C  Input file : .con   in     8
C
20    format(a20)
      print *,'Input file .con (no extension) '
      read 20, file
      lfile=len(file)
      do while (file(lfile:lfile).eq.' ')
         lfile=lfile-1
      enddo
      tempo=file(1:lfile)//'.con'
      call open_read1(8,tempo)
C
C Output file *-new.con  in  15
C
      tempo=file(1:lfile)//'-new.con'
      inquire(file=tempo,exist=qex)
      if(qex.eq..FALSE.) go to 259
      call filedel(15,tempo)
259   continue
      call open_write1(15,tempo)
C
C Output file *.txt  in  16
C
      tempo=file(1:lfile)//'.txt'
      inquire(file=tempo,exist=qex)
      if(qex.eq..FALSE.) go to 260
      call filedel(16,tempo)
260   continue
      call open_write1(16,tempo)
C
C     Read the connectivity data only once   in   9
C
      tempo='connectivity.txt'
      call open_read1(9,tempo)
C  Read total number of data
      i=0
C  Read name
5305  i=i+1
      read(9,5302,end=5306)zeot(i),r(i),nsg(i),sgn(i)
      if(nsg(i).eq.0)nsg(i)=231
C  Read number of T sites for that data
      read(9,*)nzeot(i)
C  Read multiplicity of T sites for that data
      read(9,*)(nmzeot(i,k),k=1,nzeot(i))
      do 5300 j=1,nzeot(i)
      read(9,5301)coseq(j,i)(1:40)
      lm(i)=10
      if(coseq(j,i)(37:40).eq.'    ')lm(i)=9
      if(coseq(j,i)(33:36).eq.'    ')lm(i)=8
      if(coseq(j,i)(29:32).eq.'    ')lm(i)=7
      if(coseq(j,i)(25:28).eq.'    ')lm(i)=6
      if(coseq(j,i)(21:24).eq.'    ')lm(i)=5
      if(coseq(j,i)(17:20).eq.'    ')lm(i)=4
      if(coseq(j,i)(13:16).eq.'    ')lm(i)=3
      if(coseq(j,i)(9:12).eq.'    ')lm(i)=2
      if(coseq(j,i)(5:8).eq.'    ')lm(i)=1
5300  continue
      go to 5305
5301  format(40A)
5302  format(A32,F7.4,I5,1x,A10)
5303  format(A32,21x,F7.4,I5,' eliminated')
5306  i=i-1
      ndat=i
C   initial number to memorize :
      ndat0=ndat
      close(9)
C
C     Read data in the .con file
C
      i=0
C  Read name
6305  i=i+1
      ij=0
      read(8,5302,end=6306)zeotc(i),rc(i),nsgc(i),sgnc(i)
C  Read number of T sites for that data
      read(8,*)nzeotc(i)
C
C  Care... if nzeotc > 10, discard...
      if(nzeotc(i).gt.10)then
      ij=1
      endif
C
C  Read multiplicity of T sites for that data
      read(8,*)(nmzeotc(i,k),k=1,nzeotc(i))
      do 6300 j=1,nzeotc(i)
      read(8,5301)coseqc(j,i)(1:40)
      lmc(i)=10
      if(coseqc(j,i)(37:40).eq.'   0')lmc(i)=9
      if(coseqc(j,i)(33:36).eq.'   0')lmc(i)=8
      if(coseqc(j,i)(29:32).eq.'   0')lmc(i)=7
      if(coseqc(j,i)(25:28).eq.'   0')lmc(i)=6
      if(coseqc(j,i)(21:24).eq.'   0')lmc(i)=5
      if(coseqc(j,i)(17:20).eq.'   0')lmc(i)=4
      if(coseqc(j,i)(13:16).eq.'   0')lmc(i)=3
      if(coseqc(j,i)(9:12).eq.'   0')lmc(i)=2
      if(coseqc(j,i)(5:8).eq.'   0')lmc(i)=1
6300  continue
        if(ij.eq.1)then
        write(16,5303)zeotc(i),rc(i),nsgc(i),sgnc(i)
        i=i-1
        else
        write(16,5302)zeotc(i),rc(i),nsgc(i),sgnc(i)
        endif
      go to 6305
6306  i=i-1
      ndatc=i
      close(8)
C
C  Compare coordination sequence
C
C
C  Try to identify if some data in the .con
C      are already in the connectivity.txt file
C
C
C     Now compare with data in connectivity.txt ...
C
C     compare only on the real length of the predicted sequence
C     
      do 6810 ii=1,ndatc
      do 6800 i=1,ndat
C
C        skip if not same ntype
C
      if(nzeot(i).ne.nzeotc(ii))go to 6800
C  Shorter sequence ?
      l=lmc(ii)
      if(l.gt.lm(i))l=lm(i)
C
      l=l*4
      isum=0
      do 6700 j=1,nzeotc(ii)
      do 6699 k=1,nzeot(i)
      if(coseqc(j,ii)(1:l).eq.coseq(k,i)(1:l))then
      isum=isum+1
      if(isum.eq.nzeotc(ii))go to 6801
      endif
6699  continue      
6700  continue
6800  continue
C   
C  Nothing found, then it is new, so, store it in the *-new.con
C
      write(*,*)zeotc(ii)(1:11),'                 is probably new'
      write(16,*)zeotc(ii)(1:11),'                 is probably new'
C
C    Add this new model into the list of known ones...
C
      ndat=ndat+1
C
C  ns : s if for same
      ns(ndat)=0
C
      zeot(ndat)(1:32)=zeotc(ii)(1:32)
      nzeot(ndat)=nzeotc(ii)
      r(ndat)=rc(ii)
      nsg(ndat)=nsgc(ii)
      sgn(ndat)=sgnc(ii)
      do k=1,nzeotc(ii)
      nmzeot(ndat,k)=nmzeotc(ii,k)
      coseq(k,ndat)(1:40)=coseqc(k,ii)(1:40)
      enddo
      lm(ndat)=lmc(ii)
C
      go to 6810
C
C    Something found 
C
6801  continue  
      write(*,*)zeotc(ii)(1:11),' is probably ',zeot(i)(1:11)
      write(16,*)zeotc(ii)(1:11),' is probably ',zeot(i)(1:11)
C
C
C   But what if a model is better in R or in symmetry 
C   than one already inside of the connectivity list ???
C      this info (R and SG number) has to be added 
C      inside of the connectivity list for comparison purpose...
C
C
      write(*,*)'   R factors   ',rc(ii),r(i)
      write(16,*)'   R factors   ',rc(ii),r(i)
      if(rc(ii).le.r(i))then
      write(*,*)'   but with better R, or =...',rc(ii),r(i)
      write(16,*)'   but with better R, or =...',rc(ii),r(i)
C
C   If yes, update... but only if i > ndat0
C
      if(i.gt.ndat0)then
      write(*,*)'   update made'
      write(16,*)'   update made'
C
C   also save those that were identical but with larger R
C
      ns(i)=ns(i)+1
      zeots(i,ns(i))(1:32)=zeot(i)(1:32)
      rs(i,ns(i))=r(i)
      nsgs(i,ns(i))=nsg(i)
      sgns(i,ns(i))=sgn(i)
C
      zeot(i)(1:32)=zeotc(ii)(1:32)
      nzeot(i)=nzeotc(ii)
      r(i)=rc(ii)
      nsg(i)=nsgc(ii)
      sgn(i)=sgnc(ii)
      do k=1,nzeotc(ii)
      nmzeot(i,k)=nmzeotc(ii,k)
      coseq(k,i)(1:40)=coseqc(k,ii)(1:40)
      enddo
      lm(i)=lmc(ii)
      else
      write(*,*)'   update not made'
      write(16,*)'   update not made'
      endif
C
C  This does not work yet, since the space group number
C       is not correctly added into the CIF and .con files
C
      if(nsgc(ii).gt.nsg(i))then
      write(*,*)'   and a higher symmetry...',nsgc(ii),nsg(i)
      write(16,*)'   and a higher symmetry...',nsgc(ii),nsg(i)
      endif
      endif
C
C
C
6810  continue
C   
C  Store the results in the *-new.con
C
      igood=0
      do ii=ndat0+1,ndat
      igood=igood+1
      rd(igood)=r(ii)
C      write(15,'(a32,F7.4,2x,I3,1x,a10)')zeot(ii),r(ii),nsg(ii),sgn(ii)
C      write(16,'(a32,F7.4,2x,I3,1x,a10)')zeot(ii),r(ii),nsg(ii),sgn(ii)
C      write(15,'(I3)')nzeot(ii)
C      write(15,'(20I3)')(nmzeot(ii,ij),ij=1,nzeot(ii))
      do kj=1,nzeot(ii)
C      write(15,'(40a)')coseq(kj,ii)
      enddo
      enddo
      write(16,*)
      write(16,*)'   models to save :',igood
C
C  Order according to R
C
      CALL SORT(IGOOD,RD,LL)
      write(16,*)
      WRITE(16,*)'  Models ordered according to R'
      DO I=1,IGOOD
      ii=LL(I)+ndat0
      write(15,'(a32,F7.4,2x,I3,1x,a10)')zeot(ii),r(ii),nsg(ii),sgn(ii)
      write(16,'(a32,F7.4,2x,I3,1x,a10)')zeot(ii),r(ii),nsg(ii),sgn(ii)
C
C  write also those identical solution with larger R
C
        if(ns(ii).ne.0)then
        do jj=1,ns(ii)
      write(16,'(1x,a32,F7.4,2x,I3,1x,a10)')zeots(ii,jj),rs(ii,jj),
     1nsgs(ii,jj),sgns(ii,jj)
        enddo
        endif
      write(15,'(I3)')nzeot(ii)
      write(15,'(20I3)')(nmzeot(ii,ij),ij=1,nzeot(ii))
      do kj=1,nzeot(ii)
      write(15,'(40a)')coseq(kj,ii)
      enddo
      enddo
C
C
      close(15)
      close(16)
       WRITE(*,3955)
3955  FORMAT(//'  Type any character and a RETURN to continue : ',$)
      READ(*,*)FEND
      stop
      end
*
********************************************************* 
*
      SUBROUTINE SORT(N,A,L)
C     ********************************************************
C     THE SUBROUTINE SORT APPLIES TO THE ARRAY A WITH J ELEMENTS.
C     THE INDICES OF THE ORDERED ARRAY ARE PLACED IN ARRAY L
C     TO OBTAIN THE ORDERED ARRAY REPLACE THE INDICES I OF A(I)
C     WITH I=L(I)---REF.CACM 271
C     *********************************************************
C
      PARAMETER(NGOOD=10000)
      DIMENSION IUT(10),ILT(10)
      DIMENSION A(NGOOD),L(NGOOD)
      J=N
      I=1
      M=1
      DO 10 K=1,J
10    L(K)=K
20    IF(J-I-1)140,140,30
30    IP=(J+I)/2
      ITT=L(IP)
      T=A(ITT)
      L(IP)=L(I)
      IQ=J
      K=I+1
40    IF(K.GT.IQ)GOTO 90
      LK=L(K)
      IF(A(LK).LE.T)GOTO 80
      IQ=IQ
50    IF(IQ.LT.K)GOTO 70
      LQ=L(IQ)
      IF(A(LQ).GE.T)GOTO 60
      IX=L(K)
      L(K)=L(IQ)
      L(IQ)=IX
      IQ=IQ-1
      GOTO 80
60    IQ=IQ-1
      GOTO 50
70    CONTINUE
      IQ=K-1
      GOTO 100
80    K=K+1
      GOTO 40
90    CONTINUE
100   L(I)=L(IQ)
      L(IQ)=ITT
      IF(2*IQ-I-J)120,120,110
110   ILT(M)=I
      IUT(M)=IQ-1
      I=IQ+1
      GOTO 130
120   ILT(M)=IQ+1
      IUT(M)=J
      J=IQ-1
130   M=M+1
      GOTO 20
140   IF(I-J)150,170,170
150   LI=L(I)
      LJ=L(J)
      IF(A(LI)-A(LJ))170,170,160
160   IX=L(I)
      L(I)=L(J)
      L(J)=IX
170   M=M-1
      IF(M)190,190,180
180   I=ILT(M)
      J=IUT(M)
      GOTO 20
190   CONTINUE
      RETURN
      END
*
*
*-----------------------------------------------------------------------
*
*     Subroutine open_read1 opens a file for reading, filling in the
*     supplied extension if none is supplied with the file name.
*     Allows use of system specific facilities of open statement.
*
*
*     file formatted
*
      subroutine open_read1(unit,file)
      integer unit
      character*(*) file
      open (unit,file=file,status='old')
      return
      end
*
*     Subroutine open_write1 opens a file for writing, filling in the
*     supplied extension if none is supplied with the file name.
*     Allows use of system specific facilities of open statement.
*
*     file formatted
*
      subroutine open_write1(unit,file)
      integer unit
      character*(*) file
      open (unit,file=file,status='new')
      return
      end
*
      subroutine filedel(unit,file)
      integer unit
      character*(*) file
      open (unit,file=file,status='old')
      close(unit,status='delete')
      return
      end
*
*-----------------------------------------------------------------------
